import requests
import json

import joblib
import tempfile

import pickle

##########

DQ_API_SERVER = 'https://apis.deque.app'  # ''127.0.0.1'#


class DQ_Client:
    def __init__(self):
        self.token = None

    def init(self , user_name , api_key):

        try:
            data = {"user_name": user_name , "api_key": api_key}
            resp = requests.post (DQ_API_SERVER + '/fclient/init/' ,
                                  json=data)
            function_response = resp.json ()
            if 'token' not in function_response:
                raise Exception ("Auth failed. Check your API key")

            self.token = function_response[ 'token' ]

        except requests.exceptions.ConnectionError:
            print ("Connection Error")

        return self.token

    def save_model(self , user_name , model_name , framework , persistence_mechanism , model):

        try:

            data = {'user_name': user_name , 'token': self.token , 'model_name': model_name , 'framework': framework ,
                    'persistence_mechanism': persistence_mechanism}

            file_names = [ ]
            if persistence_mechanism == "joblib":
                with tempfile.TemporaryFile () as fp:
                    joblib.dump (model , fp)
                    fp.seek (0)
                    file_names.append (('file' , (model_name , fp.read() , 'application/octet')))
            if persistence_mechanism == "pickle":
                with tempfile.TemporaryFile () as   fp:
                    pickle.dump (model , fp)
                    fp.seek (0)
                    file_names.append (('file' , (model_name , fp.read() , 'application/octet')))
            file_names.append (('data' , ('data' , json.dumps (data) , 'application/json')))


            resp = requests.post (DQ_API_SERVER + '/fclient/model/create/' ,
                                  files=file_names)

        except requests.exceptions.ConnectionError:
            print ("Connection Error")

    def load_model(self , user_name , model_name):
        try:
            f_data = {'model_name': model_name , 'user_name': user_name, 'token': self.token}
            resp0 = requests.post (DQ_API_SERVER + '/fclient/modeldetails/read/' , json=f_data)
            data0 =  resp0.json()
            persistence_mechanism = data0['persistence_mechanism']
            framework = data0['framework']
            resp1 = requests.post (DQ_API_SERVER + '/fclient/model/read/' , json=f_data)
            data = resp1.content

            with tempfile.TemporaryFile (mode='w+b') as fp:

                fp.write (data)

                fp.seek (0)
                if persistence_mechanism == "joblib":
                    model = joblib.load ( fp)
                if persistence_mechanism == "pickle":
                    model = pickle.load (fp)
            return model
        except requests.exceptions.ConnectionError:
            print ("Connection Error")




if __name__ == "__main__":
    client = DQ_Client ()

